package com.hz.pm.api.projectlib.manage;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hz.pm.api.external.MhApiClient;
import com.hz.pm.api.projectlib.entity.MhSystemReplaceInfo;
import com.hz.pm.api.projectlib.model.dto.MhSystemReplaceInfoDTO;
import com.hz.pm.api.projectlib.model.dto.MhSystemReplaceInfoVO;
import com.hz.pm.api.projectlib.model.req.MhSystemReplaceReq;
import com.hz.pm.api.projectlib.service.IMhSystemReplaceInfoService;
import com.hz.pm.api.user.helper.MhUnitCache;
import com.ningdatech.basic.model.PageVo;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * MhSystemReplaceManage
 * </p>
 *
 * @author WendyYang
 * @since 15:11 2024/8/16
 */
@Component
@RequiredArgsConstructor
public class MhSystemReplaceManage {

    private final IMhSystemReplaceInfoService mhSystemReplaceInfoService;
    private final MhUnitCache mhUnitCache;
    private final MhApiClient mhApiClient;


    @Transactional(rollbackFor = Exception.class)
    public synchronized void syncMhSystemReplaceInfo() {
        List<MhSystemReplaceInfoDTO> systemReplaceInfos = mhApiClient.querySystemReplaceInfos();
        mhSystemReplaceInfoService.remove(null);
        if (CollUtil.isEmpty(systemReplaceInfos)) {
            return;
        }
        LocalDateTime now = LocalDateTime.now();
        List<MhSystemReplaceInfo> newData = systemReplaceInfos.stream().map(w -> {
            MhSystemReplaceInfo replaceInfo = new MhSystemReplaceInfo();
            replaceInfo.setActualFinishDate(w.getFinalTime());
            replaceInfo.setBuildOrgCode(w.getCreateUnit());
            replaceInfo.setBuildOrgName(w.getBuildUnit());
            replaceInfo.setExpectCost(w.getExpectCost());
            replaceInfo.setHost(w.getHost());
            replaceInfo.setId(w.getId());
            replaceInfo.setCreateOn(now);
            replaceInfo.setIsSecret(w.getIsSecret());
            replaceInfo.setPid(w.getPid());
            replaceInfo.setPlanFinishDate(w.getExpectDate());
            replaceInfo.setPlanStartDate(w.getExpectStart());
            replaceInfo.setReplaceType(w.getAlternativeType());
            replaceInfo.setSourceName(w.getSystemName());
            replaceInfo.setSystemNum(1);
            replaceInfo.setTargetName(w.getAlternativeName());
            return replaceInfo;
        }).collect(Collectors.toList());
        mhSystemReplaceInfoService.saveBatch(newData);
    }

    public PageVo<MhSystemReplaceInfoVO> page(MhSystemReplaceReq req) {
        LambdaQueryWrapper<MhSystemReplaceInfo> query = Wrappers.lambdaQuery(MhSystemReplaceInfo.class)
                .like(StrUtil.isNotBlank(req.getBuildOrgName()), MhSystemReplaceInfo::getBuildOrgName, req.getBuildOrgName())
                .and(StrUtil.isNotBlank(req.getSystemName()), q1 -> q1.like(MhSystemReplaceInfo::getSourceName, req.getSystemName())
                        .or(q2 -> q2.like(MhSystemReplaceInfo::getTargetName, req.getSystemName())))
                .eq(StrUtil.isNotBlank(req.getReplaceType()), MhSystemReplaceInfo::getReplaceType, req.getReplaceType())
                .ge(req.getPlanFinishDateMin() != null, MhSystemReplaceInfo::getPlanFinishDate, req.getPlanFinishDateMin())
                .le(req.getPlanFinishDateMax() != null, MhSystemReplaceInfo::getPlanFinishDate, req.getPlanFinishDateMax())
                .ge(req.getPlanStartDateMin() != null, MhSystemReplaceInfo::getPlanStartDate, req.getPlanStartDateMin())
                .le(req.getPlanStartDateMax() != null, MhSystemReplaceInfo::getPlanStartDate, req.getPlanStartDateMax());
        if (req.getFinish() != null) {
            if (Boolean.TRUE.equals(req.getFinish())) {
                query.orderByDesc(MhSystemReplaceInfo::getActualFinishDate);
                query.isNotNull(MhSystemReplaceInfo::getActualFinishDate);
            } else {
                query.orderByDesc(MhSystemReplaceInfo::getPlanFinishDate);
                query.isNull(MhSystemReplaceInfo::getActualFinishDate);
            }
        } else {
            query.orderByDesc(MhSystemReplaceInfo::getPlanFinishDate);
        }
        if (req.getBuildOrgCode() != null) {
            query.in(MhSystemReplaceInfo::getBuildOrgCode, mhUnitCache.getViewChildIdsRecursion(req.getBuildOrgCode()));
        }
        Page<MhSystemReplaceInfo> page = mhSystemReplaceInfoService.page(req.page(), query);
        if (page.getTotal() == 0) {
            return PageVo.empty();
        }
        List<MhSystemReplaceInfoVO> data = BeanUtil.copyToList(page.getRecords(), MhSystemReplaceInfoVO.class);
        return PageVo.of(data, page.getTotal());
    }

}
